@@ -1,9 +1,9 @@
 =begin
-  top-level extconf.rb for Ruby-GNOME
+  top-level extconf.rb for Ruby-GNOME2
 
   $Id: extconf.rb,v 1.17 2007/10/22 12:19:17 ktou Exp $
 
-  Copyright (C) 2003-2025  Ruby-GNOME Project Team
+  Copyright (C) 2003-2005 Ruby-GNOME2 Project Team
 =end
 
 require 'English'
@@ -11,7 +11,7 @@ require 'mkmf'
 require 'fileutils'
 require 'pathname'
 
-prior_libs = [
+priorlibs = [
   "glib2",
   "cairo-gobject",
   "gobject-introspection",
@@ -19,12 +19,9 @@ prior_libs = [
   "gdk_pixbuf2",
   "pango",
   "atk",
+  "gtk2",
   "gdk3",
   "gtk3",
-  "graphene1",
-  "gdk4",
-  "gsk4",
-  "gtk4",
 ]
 
 unsupported_libraries = [
@@ -53,15 +50,15 @@ $topdir = File.expand_path($topdir)
 
 subdirs = ARGV.select{|v|  /^--/ !~ v}
 
-if subdirs.empty?
+if subdirs.size == 0
   subdirs = Dir.glob($topsrcdir+"/*/extconf.rb")
   subdirs.collect! do |subdir|
     subdir[0..$topsrcdir.size] = ""
     File.dirname(subdir)
   end
-  prior_libs &= subdirs
-  subdirs -= prior_libs
-  subdirs = prior_libs + subdirs # Change the order
+  priorlibs &= subdirs
+  subdirs -= priorlibs
+  subdirs = priorlibs + subdirs #Change the order
 end
 subdirs = subdirs.delete_if do |subdir|
   subdir.end_with?("-no-gi")
@@ -82,7 +79,6 @@ else
 end
 
 subdirs.each do |subdir|
-  puts "::group::Run extconf.rb: #{subdir}"
   STDERR.puts("#{$0}: Entering directory `#{subdir}'")
   FileUtils.mkdir_p(subdir)
   topdir = File.join(*([".."] * subdir.split(/\/+/).size))
@@ -99,11 +95,8 @@ subdirs.each do |subdir|
   else
     ignore_modules << subdir
   end
-  puts "::endgroup::"
 end
-
-puts "::group::Show top-level extconf.rb results"
-puts "-----"
+puts "\n-----"
 unless target_modules.empty?
   puts "Target libraries: #{target_modules.join(', ')}"
 end
@@ -155,6 +148,5 @@ end
 
 puts "-----"
 puts "Done."
-puts "::endgroup::"
 
 $makefile_created = true
