@@ -33,15 +33,15 @@
 #define GMAINCONTEXT2RVAL(o)               (BOXED2RVAL(o, G_TYPE_MAIN_CONTEXT))
 #define RVAL2GMAINLOOP(o)                  ((GMainLoop*)RVAL2BOXED(o, G_TYPE_MAIN_LOOP))
 #define GMAINLOOP2RVAL(o)                  (BOXED2RVAL(o, G_TYPE_MAIN_LOOP))
-#define RVAL2GPOLLFD(o)                    ((GPollFD*)RVAL2BOXED(o, G_TYPE_POLL_FD))
-#define GPOLLFD2RVAL(o)                    (BOXED2RVAL(o, G_TYPE_POLL_FD))
+#define RVAL2GPOLLFD(o)                    ((GPollFD*)RVAL2BOXED(o, G_TYPE_POLLFD))
+#define GPOLLFD2RVAL(o)                    (BOXED2RVAL(o, G_TYPE_POLLFD))
 #define RVAL2GSOURCE(o)                    ((GSource*)RVAL2BOXED(o, G_TYPE_SOURCE))
 #define GSOURCE2RVAL(o)                    (BOXED2RVAL(o, G_TYPE_SOURCE))
 #define RVAL2GTIMER(o)                     ((GTimer*)RVAL2BOXED(o, G_TYPE_TIMER))
 #define GTIMER2RVAL(o)                     (BOXED2RVAL(o, G_TYPE_TIMER))
 #define RVAL2GVALUE(o)                     ((GValue*)RVAL2BOXED(o, G_TYPE_VALUE))
 #define GVALUE2RVAL(o)                     (BOXED2RVAL(o, G_TYPE_VALUE))
-#define RVAL2GVARIANTTYPE(o)               ((GVariantType *)RVAL2BOXED(o, G_TYPE_VARIANT_TYPE))
+#define RVAL2GVARIANTTYPE(o)               (rbg_variant_type_from_ruby(o))
 #define GVARIANTTYPE2RVAL(o)               (BOXED2RVAL(o, G_TYPE_VARIANT_TYPE))
 
 #define RVAL2GIOCONDITION(o)               (RVAL2GFLAGS(o, G_TYPE_IO_CONDITION))
