@@ -1,6 +1,6 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 /*
- *  Copyright (C) 2002-2018  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2002-2019  Ruby-GNOME Project Team
  *  Copyright (C) 2002,2003  Masahiro Sakai
  *
  *  This library is free software; you can redistribute it and/or
@@ -99,20 +99,15 @@ rbgobj_class_info_fill_name(RGObjClassInfo *cinfo)
     cinfo->data_type->wrap_struct_name = cinfo->name;
 }
 
-static RGObjClassInfo *
-rbgobj_class_info_register_without_lock(GType gtype, VALUE klass)
+static rb_data_type_t *
+rbgobj_class_info_create_data_type(VALUE klass)
 {
     rb_data_type_t *data_type;
-    RGObjClassInfo *cinfo;
-    GType fundamental_type;
-    RGObjClassInfoDynamic *cinfod;
-    void *gclass = NULL;
-    VALUE c;
 
     data_type = RB_ZALLOC(rb_data_type_t);
     data_type->function.dmark = cinfo_mark;
     data_type->function.dfree = cinfo_free;
-    if (RB_TYPE_P(klass, RUBY_T_CLASS)) {
+    if (RB_TYPE_P(klass, RUBY_T_CLASS) && klass != rb_cObject) {
         VALUE p = RCLASS_SUPER(klass);
         while (p != rb_cObject) {
             if (RTYPEDDATA_P(p)) {
@@ -124,6 +119,20 @@ rbgobj_class_info_register_without_lock(GType gtype, VALUE klass)
     }
     data_type->flags = RUBY_TYPED_FREE_IMMEDIATELY;
 
+    return data_type;
+}
+
+static RGObjClassInfo *
+rbgobj_class_info_register_without_lock(GType gtype, VALUE klass)
+{
+    rb_data_type_t *data_type;
+    RGObjClassInfo *cinfo;
+    GType fundamental_type;
+    RGObjClassInfoDynamic *cinfod;
+    void *gclass = NULL;
+    VALUE c;
+
+    data_type = rbgobj_class_info_create_data_type(klass);
     c = TypedData_Make_Struct(rb_cData, RGObjClassInfo, data_type, cinfo);
     cinfo->klass = klass;
     cinfo->gtype = gtype;
@@ -368,7 +377,7 @@ rbgobj_class_info_lookup(VALUE klass)
     if (TYPE(klass) == T_CLASS) {
         VALUE super;
         super = rb_funcall(klass, id_superclass, 0);
-        return rbgobj_lookup_class(super);
+        return rbgobj_class_info_lookup(super);
     }
 
     rb_raise(rb_eRuntimeError, "can't get gobject class information");
@@ -420,6 +429,30 @@ rbgobj_gtype_to_ruby_class(GType gtype)
     return cinfo ? cinfo->klass : Qnil;
 }
 
+GType
+rbgobj_gtype_from_ruby(VALUE rb_gtype)
+{
+    ID id_gtype;
+
+    if (RB_TYPE_P(rb_gtype, RUBY_T_STRING)) {
+        GType gtype;
+        gtype = g_type_from_name(RVAL2CSTR(rb_gtype));
+        if (gtype == G_TYPE_INVALID) {
+            rb_raise(rb_eArgError,
+                     "unknown GType name: <%s>",
+                     RVAL2CSTR(rb_gtype));
+        }
+        return gtype;
+    }
+
+    CONST_ID(id_gtype, "gtype");
+    if (rb_respond_to(rb_gtype, id_gtype)) {
+        rb_gtype = rb_funcall(rb_gtype, id_gtype, 0);
+    }
+
+    return NUM2ULONG(rb_gtype);
+}
+
 VALUE
 rbgobj_define_class(GType gtype, const gchar *name, VALUE module,
                     RGMarkFunc mark, RGFreeFunc free, VALUE parent)
@@ -493,11 +526,13 @@ rbgobj_register_class(VALUE klass,
                       gboolean klass2gtype,
                       gboolean gtype2klass)
 {
-    RGObjClassInfo* cinfo = NULL;
+    rb_data_type_t *data_type = NULL;
+    RGObjClassInfo *cinfo = NULL;
     VALUE c = Qnil;
 
     if (klass2gtype) {
-        c = Data_Make_Struct(rb_cData, RGObjClassInfo, cinfo_mark, NULL, cinfo);
+        data_type = rbgobj_class_info_create_data_type(klass);
+        c = TypedData_Make_Struct(rb_cData, RGObjClassInfo, data_type, cinfo);
     }
     if (gtype2klass && !cinfo)
         cinfo = g_new(RGObjClassInfo, 1);
@@ -508,6 +543,7 @@ rbgobj_register_class(VALUE klass,
         cinfo->mark  = NULL;
         cinfo->free  = NULL;
         cinfo->flags = 0;
+        cinfo->data_type = data_type;
     }
 
     if (klass2gtype)
@@ -593,6 +629,61 @@ rbgobj_gtype_get(VALUE self)
     rb_raise(rb_eTypeError, "Not a GLib::Type");
 }
 
+static VALUE
+rg_s_try_convert(VALUE self, VALUE value)
+{
+    ID id_new;
+    CONST_ID(id_new, "new");
+
+    if (NIL_P(value))
+        return Qnil;
+
+    if (RVAL2CBOOL(rb_obj_is_kind_of(value, RG_TARGET_NAMESPACE)))
+        return value;
+
+    if (RVAL2CBOOL(rb_obj_is_kind_of(value, rb_cInteger))) {
+        GType gtype = NUM2ULONG(value);
+        if (!g_type_name(gtype))
+            return Qnil;
+        return rb_funcall(self, id_new, 1, value);
+    }
+
+    if (RB_TYPE_P(value, RUBY_T_SYMBOL)) {
+        value = rb_sym2str(value);
+    }
+
+    if (RB_TYPE_P(value, RUBY_T_STRING)) {
+        GType gtype = g_type_from_name(RVAL2CSTR(value));
+        if (gtype == G_TYPE_INVALID) {
+            return Qnil;
+        }
+        return rb_funcall(self, id_new, 1, value);
+    }
+
+    if (RVAL2CBOOL(rb_obj_is_kind_of(value, rb_cClass))) {
+        for (; !NIL_P(value); value = rb_funcall(value, id_superclass, 0)) {
+            RGObjClassInfo *cinfo;
+            VALUE data = rb_hash_aref(klass_to_cinfo, value);
+
+            if (NIL_P(data))
+                continue;
+
+            if (RTYPEDDATA_P(data)) {
+                TypedData_Get_Struct(data,
+                                     RGObjClassInfo,
+                                     RTYPEDDATA_TYPE(data),
+                                     cinfo);
+            } else {
+                Data_Get_Struct(data, RGObjClassInfo, cinfo);
+            }
+            return rb_funcall(self, id_new, 1, ULONG2NUM(cinfo->gtype));
+        }
+        return Qnil;
+    }
+
+    return Qnil;
+}
+
 static VALUE
 rg_initialize(VALUE self, VALUE type)
 {
@@ -918,6 +1009,8 @@ Init_gobject_gtype(void)
 
     RG_TARGET_NAMESPACE = rb_define_class_under(mGLib, "Type", rb_cObject);
 
+    RG_DEF_SMETHOD(try_convert, 1);
+
     rb_define_alias(CLASS_OF(RG_TARGET_NAMESPACE), "[]", "new");
     RG_DEF_METHOD(initialize, 1);
     RG_DEF_METHOD(inspect, 0);
