@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2017  Ruby-GNOME2 Project Team
+# Copyright (C) 2005-2019  Ruby-GNOME Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -114,20 +114,7 @@ module GLib
   end
 end
 
-
-base_dir = Pathname.new(__FILE__).dirname.dirname.expand_path
-vendor_dir = base_dir + "vendor" + "local"
-if vendor_dir.exist?
-  require "cairo"
-end
-
-GLib.prepend_dll_path(vendor_dir + "bin")
-begin
-  major, minor, _ = RUBY_VERSION.split(/\./)
-  require "#{major}.#{minor}/glib2.so"
-rescue LoadError
-  require 'glib2.so'
-end
+require "glib2.so"
 
 module GLib
   module MetaInterface
@@ -161,23 +148,50 @@ module GLib
 
 
   class Enum
-    def _dump(limit)
-      Marshal.dump(to_i, limit)
+    class << self
+      def try_convert(value)
+        if value.is_a?(self)
+          value
+        else
+          find(value)
+        end
+      end
+
+      def _load(obj)
+        new(Marshal.load(obj))
+      end
     end
 
-    def self._load(obj)
-      new(Marshal.load(obj))
+    def _dump(limit)
+      Marshal.dump(to_i, limit)
     end
   end
 
 
   class Flags
-    def _dump(limit)
-      Marshal.dump(to_i, limit)
+    class << self
+      def try_convert(value)
+        case value
+        when self
+          value
+        when Integer, String, Symbol, Array
+          begin
+            new(value)
+          rescue ArgumentError
+            nil
+          end
+        else
+          nil
+        end
+      end
+
+      def _load(obj)
+        new(Marshal.load(obj))
+      end
     end
 
-    def self._load(obj)
-      new(Marshal.load(obj))
+    def _dump(limit)
+      Marshal.dump(to_i, limit)
     end
 
     # FIXME
@@ -269,8 +283,9 @@ GLib::Log.set_log_domain(GLib::Object::LOG_DOMAIN)
 GLib::Log.set_log_domain(GLib::Thread::LOG_DOMAIN)
 GLib::Log.set_log_domain(GLib::Module::LOG_DOMAIN)
 
-require 'glib2/version'
+require "glib2/version"
 require "glib2/regex"
+require "glib2/deprecated"
 =begin
 Don't we need this?
 ObjectSpace.define_finalizer(GLib) {
