@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 #
-# Copyright (C) 2015  Ruby-GNOME2 Project Team
+# Copyright (C) 2015-2019  Ruby-GNOME Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -36,7 +36,7 @@ $LOAD_PATH.unshift(mkmf_gnome2_dir.to_s)
 module_name = "gtk3"
 package_id = "gtk+-3.0"
 
-require "mkmf-gnome2"
+require "mkmf-gnome"
 
 depended_packages = [
   "glib2",
@@ -62,12 +62,10 @@ unless check_cairo(:top_dir => top_dir)
   exit(false)
 end
 
-setup_windows(module_name, base_dir)
-
 unless required_pkg_config_package(package_id,
                                    :alt_linux => "libgtk+3-devel",
                                    :debian => "libgtk-3-dev",
-                                   :redhat => "gtk3-devel",
+                                   :redhat => "pkgconfig(#{package_id})",
                                    :arch_linux => "gtk3",
                                    :homebrew => "gtk+3",
                                    :macports => "gtk3",
