@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2015  Ruby-GNOME2 Project Team
+# Copyright (C) 2006-2018  Ruby-GNOME2 Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -14,15 +14,10 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-require "gobject-introspection"
 require "atk"
 require "gdk3"
 require "gio2"
 
-if /cygwin|mingw|mswin/ === RUBY_PLATFORM
-  require "rsvg2"
-end
-
 require "gtk3/loader"
 
 module Gtk
@@ -60,7 +55,6 @@ module Gtk
         remove_method(:const_missing)
         remove_method(:method_missing)
       end
-      Gio.init if Gio.respond_to?(:init)
       Gdk.init if Gdk.respond_to?(:init)
       loader = Loader.new(self, argv)
       loader.load
