@@ -4,7 +4,10 @@ ruby_gnome2_base = File.join(File.dirname(__FILE__), "..", "..")
 ruby_gnome2_base = File.expand_path(ruby_gnome2_base)
 
 glib_base = File.join(ruby_gnome2_base, "glib2")
+gobject_introspection_base = File.join(ruby_gnome2_base, "gobject-introspection")
+gio_base = File.join(ruby_gnome2_base, "gio2")
 atk_base = File.join(ruby_gnome2_base, "atk")
+cairo_gobject_base = File.join(ruby_gnome2_base, "cairo-gobject")
 pango_base = File.join(ruby_gnome2_base, "pango")
 gdk_pixbuf_base = File.join(ruby_gnome2_base, "gdk_pixbuf2")
 gtk_base = File.join(ruby_gnome2_base, "gtk2")
@@ -13,15 +16,21 @@ gtk_source_view2_base = File.join(ruby_gnome2_base, "gtksourceview2")
 $LOAD_PATH.unshift(glib_base)
 require 'test/glib-test-init'
 
-[glib_base, atk_base, pango_base, gdk_pixbuf_base, gtk_base,
- gtk_source_view2_base].each do |target|
-  if system("which make > /dev/null")
+[
+  [glib_base, "glib2"],
+  [gobject_introspection_base, "gobject-introspection"],
+  [gio_base, "gio2"],
+  [atk_base, "atk"],
+  [cairo_gobject_base, "cairo-gobject"],
+  [pango_base, "pango"],
+  [gdk_pixbuf_base, "gdk_pixbuf2"],
+  [gtk_base, "gtk2"],
+  [gtk_source_view2_base, "gtksourceview2"],
+].each do |target, module_name|
+  if File.exist?("#{target}/Makefile") and system("which make > /dev/null")
     `make -C #{target.dump} > /dev/null` or exit(1)
   end
-  $LOAD_PATH.unshift(File.join(target, "ext", File.basename(target)))
-  $LOAD_PATH.unshift(File.join(target, "src"))
-  $LOAD_PATH.unshift(File.join(target, "src", "lib"))
-  $LOAD_PATH.unshift(File.join(target))
+  $LOAD_PATH.unshift(File.join(target, "ext", module_name))
   $LOAD_PATH.unshift(File.join(target, "lib"))
 end
 
